red <- paste("<font color=\"red\">")
blue <- paste("<font color=\"blue\">")
green <- paste("<font color=\"green\">")
black <- paste("<font color=\"black\">")
space <- "&nbsp;&nbsp;&nbsp;"
source("sp.r")
sourceCpp("backmle.cpp")
library(numDeriv)

shinyServer(function(input, output,session) {

    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b,a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(gotLibs()[1]) library(ggplot2)
        if(gotLibs()[2]) library(grid)    
        if(!gotLibs()[3]) return(paste(red,"<h4>Warning: R libraries ggplot2 and grid not found, will use base graphics</h4>"))
        return("")       
    })
    
    genDataOwn <- function(nB,nS,nBy,mu,sig,A,B,fun) {
         eval(parse(text = paste("mydf <- function(x)",fun,sep="")))
 
         M <- max(mydf(seq(A,B,length=100)))
      
         xS <- rnorm(as.numeric(nS),as.numeric(mu),as.numeric(sig))
         xS <- xS[xS>A]
         xS <- xS[xS<B]
         if(length(xS)<as.numeric(nS)) {
             repeat {
                xS <- c(xS,rnorm(as.numeric(nS),as.numeric(mu),as.numeric(sig)))
                xS <- xS[xS>A]
                xS <- xS[xS<B]
                if(length(xS)>=as.numeric(nS)) {
                    xS <- xS[1:as.numeric(nS)]
                    break
                }
             }
         }
         xB <- rep(0,as.numeric(nB))
         for(i in 1:as.numeric(nB))
             repeat {
                x <- runif(1,A,B)
                if(runif(1)<mydf(x)/M) {
                    xB[i] <- x
                    break
                }    
         }
         yB <- rep(0,as.numeric(nBy))
         for(i in 1:as.numeric(nBy)) {
                repeat {
                      x <- runif(1,A,B)
                      if(runif(1)<mydf(x)/M) {
                         yB[i] <- x
                         break
                     }    
               } 
               y <- sort(yB)
         }     
         x <- sort(c(xB,xS))    
         list(x=x,y=y)
    }
    
    data <- eventReactive(input$button, {
        A <- as.numeric(input$A)
        B <- as.numeric(input$B)    
        if(input$whichData=="Upload your own") {
             inFile1 <- input$file1
             if (is.null(inFile1)) return(NULL)
             x<-as.numeric(read.csv(inFile1$datapath)[[1]])
             inFile2 <- input$file2
             if (is.null(inFile2)) return(NULL)
             y<-as.numeric(read.csv(inFile2$datapath)[[1]])
             return(list(x=sort(x),y=sort(y)))
        }

        if(input$Model=="Exponential") {
            x<-semiData(as.numeric(input$nB),as.numeric(input$nS),"Exponential",
                 c(as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$lambda)),A,B)
            y<-semiData(as.numeric(input$nBy),0,"Exponential",c(as.numeric(input$mu),
                     as.numeric(input$sig),as.numeric(input$lambda)),A,B)
        }        
        if(input$Model=="Flat") {
            x<-semiData(as.numeric(input$nB),as.numeric(input$nS),"Flat",
                      c(as.numeric(input$mu),as.numeric(input$sig)),A,B)
            y<-semiData(as.numeric(input$nBy),0,"Flat",
                     c(as.numeric(input$mu),as.numeric(input$sig)),A,B)
        }    
        if(input$Model=="Beta") {
            x<-semiData(as.numeric(input$nB),as.numeric(input$nS),"Beta",
                         c(as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$pbeta1),as.numeric(input$pbeta2)))    
            y<-semiData(as.numeric(input$nBy),0,"Beta", 
                                 c(as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$pbeta1),as.numeric(input$pbeta2)))    
        }
        if(input$whichData=="Enter your own background function") {
              return(genDataOwn(input$nB,input$nS,input$nBy,input$mu,input$sig,A,B,input$backfun))
        }         
                               
        list(x=x,y=y)    
   })     
       
    semiFit <- reactive( {
        A <- as.numeric(input$A)
        B <- as.numeric(input$B)

        n <- length(data()$x)        
        alpha <- ifelse(input$whichData!="Upload your own",as.numeric(input$nS)/n,as.numeric(input$nS1)/n)
        mu <- ifelse(input$whichData!="Upload your own",as.numeric(input$mu),as.numeric(input$mu))
        sig <- ifelse(input$whichData!="Upload your own",as.numeric(input$sig),as.numeric(input$sig))        
        if(input$fitWhat=="Number of Signal events only") fitWhat<-"Mix"
        if(input$fitWhat=="Number of Signal events and Signal Location") fitWhat<-"MixLoc"
        if(input$fitWhat=="All three") fitWhat<-"All"
        sp2EM.fit(x=data()$x,y=data()$y,p=c(alpha,mu,sig),fitWhat=fitWhat,A=A,B=B)
    })
                 
    parFit <- reactive({
        A <- as.numeric(input$A)
        B <- as.numeric(input$B)    
        if(input$fitWhat=="Number of Signal events only") fitWhat<-"Mix"
        if(input$fitWhat=="Number of Signal events and Signal Location") fitWhat<-"MixLoc"
        if(input$fitWhat=="All three") fitWhat<-"All"
        n <- as.numeric(input$nB)+as.numeric(input$nS)
        alpha <- as.numeric(input$nS)/n
        if(input$whichData=="Enter your own background function") {
                out <- par2EM.fit(x=data()$x,y=data()$y,p=c(alpha,as.numeric(input$mu),
                       as.numeric(input$sig)),fitWhat=fitWhat,Model="Other",dBack=input$backfun,A=A,B=B)          
        }    
        else {
            if(input$Model=="Exponential") { 
                out <- par2EM.fit(x=data()$x,y=data()$y,p=c(alpha,as.numeric(input$mu),
                   as.numeric(input$sig),as.numeric(input$lambda)),fitWhat=fitWhat,Model="Exponential",A=A,B=B)    
            }    
            if(input$Model=="Flat") { 
                out <- par2EM.fit(x=data()$x,y=data()$y,p=c(alpha,as.numeric(input$mu),
                      as.numeric(input$sig)),fitWhat=fitWhat,Model="Flat",A=A,B=B) 
            }    
            if(input$Model=="Beta")  {
                out <- par2EM.fit(x=data()$x,y=data()$y,p=c(alpha,as.numeric(input$mu),
                       as.numeric(input$sig),as.numeric(input$pbeta1),as.numeric(input$pbeta2)),fitWhat=fitWhat,Model="Beta",A=0,B=1)          
            }
        }    
        out
    })    

    polyFit <- reactive( {
        A <- as.numeric(input$A)
        B <- as.numeric(input$B)
        n <- length(data()$x)        
        alpha <- ifelse(input$whichData!="Upload your own",as.numeric(input$nS)/n,as.numeric(input$nS1)/n)
        mu <- ifelse(input$whichData!="Upload your own",as.numeric(input$mu),as.numeric(input$mu))
        sig <- ifelse(input$whichData!="Upload your own",as.numeric(input$sig),as.numeric(input$sig))        
        if(input$fitWhat=="Number of Signal events only") fitWhat<-"Mix"
        if(input$fitWhat=="Number of Signal events and Signal Location") fitWhat<-"MixLoc"
        if(input$fitWhat=="All three") fitWhat<-"All"
        poly2EM.fit(x=data()$x,y=data()$y,p=c(alpha,mu,sig),fitWhat=fitWhat,A=A,B=B)
    })
    
    drawExamples <- function() {
          x<-seq(0,1,length=250)
          y1 <- 2*exp(-2*x)/0.864664
          y2 <- dgamma(x,2,4)/pgamma(1,2,4)
          y3 <- (dexp(x,1)+1)/1.63212
          y4 <- ((-((x-1/4)^2+0.02)*(x-0.85)^2+0.06)/0.06)/0.8246181
          data.frame(x=rep(x,4),y=c(y1,y2,y3,y4),Example=as.factor(rep(1:4,each=250)))
    }
    
    output$introtext <- renderText( {
          if(input$button) return("")
          if(input$whichData=="Enter your own background function") {
                  a <- "<h5>write function for background shape in R notation (similar to C++)</h5>"
                  a <- c(a,"<h5>function need not be normalize</h5>")
                  a <- c(a,"<h5>Example 1: exp(-2*x) generates data from an exponenial distribution with rate 2
                        (truncated to interval [A,B])</h5>")
                  a <- c(a,"<h5>Example 2: dgamma(x,2,4) generates data from a gamma distribution with parameters 2 and 4</5>")      
                  a <- c(a,"<h5>Example 3: dexp(x,1)+1 generates data from a 50-50 mixture 
                            of an exponential distribution rate 1 and a uniform distribution</5>")      
                  a <- c(a,"<h5>Example 4: (-((x-1/4)^2+0.02)*(x-0.85)^2+0.06)/0.06 generates data from
                        a fourth degree polynomial</h5>")
                 return(a)
          }
          if(input$whichData=="Upload your own") {
                  return("<h5>Data should be in two ASCII files, one for sample with signal and one for pure background sample</h5>") 
          }                  
          a <- "<h5><strong>Explanation of App</strong></h5>"
          a <- c(a,"<h5> This app illustrates the semiparametric fitting method described in the paper 
                <a href=\"http://inspirehep.net/record/1081445\">Estimating a Signal In the Presence of an Unknown Background</a></h5>")
          a <- c(a,"<h5> After changing any of the setup choices on the left make sure to click the <font color=\"red\">Go!<font color=\"black\"> button
                  because not all results are updated otherwise</h5>")
          a <- c(a,"<h5>When making several changes in the problem setup it might be good to first reload the app</h5>")
          a <- c(a,"<h5> The app allows random generation of examples or use of your own data</h5>")
          a <- c(a,"<h5> Confidence intervals are found either via the large sample theory of maximum likelihood or via the statistical bootstrap </h5>")
          a <- c(a,"<h5> Various goodness of fit measures are calculated: &Chi;<sup>2</sup> (either based on large sample theory or on MC), Kolmogorov-Smirnov and Anderson-Darling </h5>")
          a <- c(a,"<h5> One can fit for either the number of signal events only, the number of signal events 
                and the signal location, or all three. This last option sometimes fails to converge to good solution.</h5>")
          a <- c(a,"<h5> As a comparison the background can also be fit using a mixture of Bernstein polynomials
                         with the degree chosen via a sequential likelihood ratio test and the fit done using the EM algorithm</h5>") 
          a <- c(a,"<h5> For random data: </h5>")
          a <- c(a,"<h5> available background models are truncated exponential, uniform(flat) or beta </h5>")
          a <- c(a,"<h5> after changing the parameters it might be necessary to hit the Go! button to update the graphs</h5>")
          a <- c(a,"<h5> the true background model is also used (with parameters fit to the data), so it is possible to compare the answer
          from the semiparametric method to the \"gold standard\" , the perfectly correct model</h5>")
          a <- c(a,"<h5> For random data one can run a mini MC study to see the performance of the method </h5>")
          a <- c(a,"<h5> Both the bootstrap and the mini MC take a while to run, so be patient!</h5>")
          a <- c(a,paste("<h5>",red,"this app is brand new and likely full of bugs, if you run into any please let me know</h5>"))
          a
    })          
    
    output$text1 <- renderText( {
         zSemi <- semiFit()$par
         if(input$whichData!="Upload your own")    
            zPar <- parFit()$par
         zPoly <- polyFit()$par
         a <- paste("<tr><td>&nbsp;</td><th align=\"center\">",red,"Semiparametric Method</th>
                                       <th align=\"center\">",space,green,"Polynomial Background</th>")
         if(input$whichData!="Upload your own")
              a<- paste(a,"<th align=\"center\">",space,blue,"Correct Background</th>")                         
         a<- paste(a,"</tr>")      
         a[2] <- paste("<tr><td align=\"left\">Estimated number of Signal Events</td>
                                 <td align=\"center\">",round(zSemi[1,1],1),"&plusmn;",round(zSemi[1,2],1),"</td>
                                 <td align=\"center\">",round(zPoly[1,1],1),"&plusmn;",round(zPoly[1,2],1),"</td>")
         if(input$whichData!="Upload your own")                                
                       a[2] <- paste(a[2],"<td align=\"center\">",round(zPar[1,1],1),"&plusmn;",round(zPar[1,2],1),"</td>")
         a[2] <- paste(a[2],"</tr>")                                                       
                                                           
         if(input$fitWhat=="Number of Signal events only") return(c("<table  width=\"500\">",a,"</table>"))
                  a[3] <- paste("<tr><td align=\"left\">Estimated Signal Location","</td>
                                    <td align=\"center\">",round(zSemi[2,1],4),"&plusmn;",round(zSemi[2,2],4),"</td>
                                    <td align=\"center\">",round(zPoly[2,1],4),"&plusmn;",round(zPoly[2,2],4),"</td>")
         if(input$whichData!="Upload your own")
                  a[3] <- paste(a[3],"<td align=\"center\">",round(zPar[2,1],4),"&plusmn;",round(zPar[2,2],4),"</td>") 
         a[3] <- paste(a[3],"</tr>")         
         if(input$fitWhat=="Number of Signal events and Signal Location") return(c("<table  width=\"500\">",a,"</table>"))        
                  a[4] <- paste("<tr><td align=\"left\">Estimated Signal Width",space,"</td>
                                    <td align=\"center\">",round(zSemi[3,1],4),"&plusmn;",round(zSemi[3,2],4),"</td>
                                    <td align=\"center\">",round(zPoly[3,1],4),"&plusmn;",round(zPoly[3,2],4),"</td>")
         if(input$whichData!="Upload your own")                 
                    a[4] <- paste(a[4],"<td align=\"center\">",round(zPar[3,1],4),"&plusmn;",round(zPar[3,2],4),"</td>")   
              a[4] <- paste(a[4],"</tr>")                       
                            
         c("<table width=\"500\">" ,a,"</table>") 
    })        
   
    output$plot1 <- renderPlot({
        if(!input$button & input$whichData=="Enter your own background function") {
            plt <- ggplot(data=drawExamples(),aes(x,y,colour=Example))+
                   geom_line(aes(x,y),size=1.2)
            return(plt)
        }    
        A <- as.numeric(input$A)
        B <- as.numeric(input$B)    
  
        fitSemi <- as.data.frame(semiFit()[1:4])  
        fitSemiPureBack <- as.data.frame(semiFit()[5:8])
        fitPoly <- as.data.frame(polyFit()[1:4]) 
        fitPolyPureBack <- as.data.frame(polyFit()[5:8])
        if(input$whichData!="Upload your own") {
            fitPar <- data.frame(parFit()[1:4])           
            fitParPureBack <- data.frame(parFit()[5:8])                       
        }  
        if(!gotLibs()[3]) {
            par(mfrow=c(1,2))
            hist(fitSemi$x,breaks=100,freq=F,main="",xlab="x")
            if(input$whichData!="Upload your own") 
                   lines(fitPar$x,fitPar$xf,col="blue",lwd=2)                       
            lines(fitPoly$x,fitPoly$xf,col="green",lwd=2)  
            lines(fitSemi$x,fitSemi$xf,col="red",lwd=2) 
            hist(fitSemi$Tx,breaks=100,freq=F,main="",xlab="Transformed x")
            lines(fitSemi$Tx,fitSemi$Txf,col="red",lwd=2)                
            if(input$whichData!="Upload your own") 
                   lines(fitPar$Tx,fitPar$Txf,col="blue",lwd=2)
            lines(fitPoly$Tx,fitPoly$Txf,col="green",lwd=2)                     
        }
        else {
            bw <- diff(range(fitSemi$x))/100
            plt1 <- ggplot(data=fitSemi,aes(x))+
                  geom_histogram(aes(y = ..density..),color="black",fill="white", binwidth = bw)+
                  geom_line(data=fitSemi[(0.05*nrow(fitSemi)):(0.95*nrow(fitSemi)),],aes(x,xf),color="red")+
                  geom_line(data=fitPoly,aes(x,xf),color="dark green")+
                  xlab("Original Data")+ylab("")
            if(input$whichData!="Upload your own") 
                plt1 <- plt1 + geom_line(data=fitPar,aes(x,xf),color="blue")
                  
            bw <- diff(range(fitSemi$Tx))/100  
            plt2 <- ggplot(data=fitSemi,aes(Tx))+
                  geom_histogram(aes(y = ..density..),color="black",fill="white", binwidth = bw)+
                  geom_line(data=fitSemi,aes(Tx,Txf),colour="red")+
                  geom_line(data=fitPoly,aes(Tx,Txf),colour="dark green")+
                  xlab("Transformed Data")+ylab("")
            if(input$whichData!="Upload your own")
                plt2 <- plt2 + geom_line(data=fitPar,aes(Tx,Txf),colour="blue") 
            
            bw <- diff(range(fitSemiPureBack$y))/100
            plt3 <- ggplot(data=fitSemiPureBack,aes(y))+
                  geom_histogram(aes(y = ..density..),color="black",fill="white", binwidth = bw)+
                  geom_line(data=fitSemiPureBack[(0.05*nrow(fitSemiPureBack)):(0.95*nrow(fitSemiPureBack)),],aes(y,yf),color="red")+
                  geom_line(data=fitPolyPureBack,aes(y,yf),color="dark green") +   
                  xlab("Original Data, Pure Background Sample")+ylab("")
            if(input$whichData!="Upload your own") 
                  plt3 <- plt3 + geom_line(data=fitParPureBack,aes(y,yf),color="blue")
                  
            bw <- diff(range(fitSemiPureBack$Ty))/100  
            plt4 <- ggplot(data=fitSemiPureBack,aes(Ty))+
                geom_histogram(aes(y = ..density..),color="black",fill="white", binwidth = bw)+
                geom_line(data=fitSemiPureBack,aes(Ty,Tyf),colour="red")+
                geom_line(data=fitPolyPureBack,aes(Ty,Tyf),colour="dark green")+             
                xlab("Transformed Data, Pure Background Sample")+ylab("")
            if(input$whichData!="Upload your own")
                plt4 <- plt4 + geom_line(data=fitParPureBack,aes(Ty,Tyf),colour="blue")

            pushViewport(viewport(layout = grid.layout(2, 2)))
            print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
            print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
            print(plt3, vp = viewport(layout.pos.row = 2, layout.pos.col = 1))
            print(plt4, vp = viewport(layout.pos.row = 2, layout.pos.col = 2))
        }    
    })
    
    output$text2 <- renderText( {
        if(input$doGof=="No") return("")
        if(input$whichData!="Included Background Models") 
                  return("<h5>Currently implemented for included background models only</h5>")
         nbins <- 50 
         A <- as.numeric(input$A)
         B <- as.numeric(input$B)             
         Model <- ifelse(input$whichData=="Included Background Models",input$Model,"Other")
         if(input$whichData=="Included Background Models") pStart <- parFit()$par[,1]
         else pStart <- semiFit()$par[,1]
         pStart[1]<-pStart[1]/(as.numeric(input$nB)+as.numeric(input$nS))     
          
         which <- c(T,F,F)
         if(input$whichGof=="All methods") which<-c(T,T,T)

         gofInfo <- gof(data()$x,data()$y,nbins=nbins,Model=Model,p=pStart,A=A,B=B,which=which) 
 
         a <- paste("<tr><td>&nbsp;</td><th>",red,space,space,"Semiparametric Method</th>")
         if(which[2]) 
              a <- paste(a,"<th>",space,space,space,blue,"Correct Background Model</th>")
         if(which[3]) 
              a <- paste(a,"<th>",space,space,space,green,"Polynomial Background Fit</th>")
         a <- paste(a,"</tr>")     
         
         a[2] <- paste("<th>&Chi;<sup>2</sup> test (Theory):</th><td>",space,space,round(gofInfo[2,1],1)," on",
                        round(gofInfo[2,3])," df (p-val=",round(gofInfo[2,2],4),")</td>")
         if(which[2]) 
              a[2] <- paste(a[2],"<td>",space,space,round(gofInfo[1,1],1)," on",
                        round(gofInfo[1,3])," df (p-val=",round(gofInfo[1,2],4),")</td>")
         if(which[3]) 
              a[2] <- paste(a[2],"<td>",space,space,round(gofInfo[9,1],1)," on",
                        round(gofInfo[9,3])," df (p-val=",round(gofInfo[9,2],4),")</td>")
         a[2] <- paste(a[2],"</tr>")                          
         
         a[3] <- paste("<th>&Chi;<sup>2</sup> test (MC):</th><td>",space,space,
                      "p-val=",round(gofInfo[4,2],4),"</td>")
         if(which[2]) 
              a[3] <- paste(a[3],"<td>",space,space,"p-val=",round(gofInfo[3,2],4),"</td>")
         if(which[3]) 
              a[3] <- paste(a[3],"<td>",space,space,"p-val=",round(gofInfo[10,2],4),"</td>")
         a[3] <- paste(a[3],"</tr>")                                   
         
         a[4] <- paste("<th>KS:</th><td>",space,space,"p-val=",round(gofInfo[6,2],4),"</td>")
         if(which[2]) 
              a[4] <- paste(a[4],"<td>",space,space,"p-val=",round(gofInfo[5,2],4),"</td>")
         if(which[3]) 
              a[4] <- paste(a[4],"<td>",space,space,"p-val=",round(gofInfo[11,2],4),"</td>")
         a[4] <- paste(a[4],"</tr>")                                   
         
         a[5] <- paste("<th>AD:</th><td>",space,space,"p-val=",round(gofInfo[8,2],4),"</td>")
         if(which[2]) 
              a[5] <- paste(a[5],"<td>",space,space,"p-val=",round(gofInfo[7,2],4),"</td>")
         if(which[3]) 
              a[5] <- paste(a[5],"<td>",space,space,"p-val=",round(gofInfo[12,2],4),"</td>")
         a[5] <- paste(a[5],"</tr>")                                   

         c("<table>" ,a,"</table>") 
                    
    }) 
             
    bootstrap <- eventReactive(input$button, {
        A <- as.numeric(input$A)
        B <- as.numeric(input$B)

        which <- c(T,F,F)
        if(input$whichBoot=="All methods") which<-c(T,T,T)
        if(input$whichData=="Upload your own") which[2]<-F

        n <- length(data()$x)        
        alpha <- ifelse(input$whichData!="Upload your own",as.numeric(input$nS)/n,as.numeric(input$nS1)/n)
        mu <- ifelse(input$whichData!="Upload your own",as.numeric(input$mu),as.numeric(input$mu))
        sig <- ifelse(input$whichData!="Upload your own",as.numeric(input$sig),as.numeric(input$sig))                    

        if(input$fitWhat=="Number of Signal events only") fitWhat<-"Mix"
        if(input$fitWhat=="Number of Signal events and Signal Location") fitWhat<-"MixLoc"
        if(input$fitWhat=="All three") fitWhat<-"All"
        x<-data()$x
        y<-data()$y
        
        progress <- shiny::Progress$new(session, min=1, max=15)
        on.exit(progress$close())
        progress$set(value = 0,message="0 % done") 
        zSemiBoot <- matrix(0,as.numeric(input$nBoot),3)     
        zParBoot <- matrix(0,as.numeric(input$nBoot),3)                  
        zPolyBoot <- matrix(0,as.numeric(input$nBoot),3)                  
        for(i in 1:as.numeric(input$nBoot)) {
             xstar <- sample(x,size=length(x),replace=T)
             ystar <- sample(y,size=length(y),replace=T)
             if(which[1])
                 zSemiBoot[i, ] <- sp2EM.fit(xstar,ystar,p=c(alpha,mu,sig),fitWhat,A=A,B=B)$par[,1]
             if(which[2]) {                    
                if(input$whichData=="Enter your own background function")               
                      zParBoot[i, ]<- par2EM.fit(xstar,ystar,p=c(alpha,mu,sig,as.numeric(input$lambda)),
                                fitWhat,Model="Other",dBack=input$backfun,A=A,B=B)$par[1:3,1]
                else {                
                    if(input$Model=="Exponential") 
                        zParBoot[i, ]<- par2EM.fit(xstar,ystar,p=c(alpha,mu,sig,as.numeric(input$lambda)),
                                fitWhat,Model="Exponential",A=A,B=B)$par[1:3,1]
                    if(input$Model=="Flat") 
                        zParBoot[i, ]<- par2EM.fit(xstar,ystar,p=c(alpha,mu,sig),
                                fitWhat,Model="Flat",A=A,B=B)$par[1:3,1]                        
                    if(input$Model=="Beta") 
                        zParBoot[i, ]<- par2EM.fit(xstar,ystar,p=c(alpha,mu,sig,as.numeric(input$pbeta1),as.numeric(input$pbeta2)),
                                fitWhat,Model="Beta",A=0,B=1)$par[1:3,1]
                }                 
             }           
             if(which[3])         
                zPolyBoot[i, ] <- poly2EM.fit(xstar,ystar,p=c(alpha,mu,sig),fitWhat,A=A,B=B)$par[,1]
             if( (i/as.numeric(input$nBoot)*100)%%10==0) 
                    progress$set(value = i/as.numeric(input$nBoot),
                                message=paste(i/as.numeric(input$nBoot)*100,"% done",sep=""))      
        }
        return(list(zSemiBoot,zParBoot,zPolyBoot))

   }) 
   output$bootplot <- renderPlot({
          zSemiBoot <- bootstrap()[[1]]
          zParBoot <- bootstrap()[[2]]
          zPolyBoot <- bootstrap()[[3]]
          which <- c(T,F,F)
          if(input$whichBoot=="All methods") which<-c(T,T,T)
          if(!which[2] & !which[3]) {
              dta <- data.frame(Mix=zSemiBoot[,1],Location=zSemiBoot[,2],Width=zSemiBoot[,3])
              plot1 <- ggplot(dta,aes(x=Mix)) + geom_histogram(color="black",fill="white")+xlab("Number of Signal Events")
              plot2 <- ggplot(dta,aes(x=Location)) + geom_histogram(color="black",fill="white")
              plot3 <- ggplot(dta,aes(x=Width)) + geom_histogram(color="black",fill="white")
          }
          if(which[2] & !which[3]) {
              dta <- data.frame(Mix=c(zSemiBoot[,1],zParBoot[,1]),
                           Location=c(zSemiBoot[,2],zParBoot[,2]),
                           Width=c(zSemiBoot[,3],zParBoot[,3]),
                           which=rep(c("Semiparametric","Correct Model"),each=as.numeric(input$nBoot))) 
              plot1 <- ggplot(dta,aes(x=Mix)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2)+
                    xlab("Number of Signal Events")
              plot2 <- ggplot(dta,aes(x=Location)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2)
              plot3 <- ggplot(dta,aes(x=Width)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2)
          }             
          if(!which[2] & which[3]) {
              dta <- data.frame(Mix=c(zSemiBoot[,1],zPolyBoot[,1]),
                           Location=c(zSemiBoot[,2],zPolyBoot[,2]),
                           Width=c(zSemiBoot[,3],zPolyBoot[,3]),
                           which=rep(c("Semiparametric","Polynomial Background Fit"),each=as.numeric(input$nBoot))) 
              plot1 <- ggplot(dta,aes(x=Mix)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "dark green", alpha = 0.2)+
                    xlab("Number of Signal Events")
              plot2 <- ggplot(dta,aes(x=Location)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "dark green", alpha = 0.2)
              plot3 <- ggplot(dta,aes(x=Width)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "dark green", alpha = 0.2)
         }             
         if(which[2] & which[3]) {
              dta <- data.frame(Mix=c(zSemiBoot[,1],zParBoot[,1],zPolyBoot[,1]),
                           Location=c(zSemiBoot[,2],zParBoot[,2],zPolyBoot[,2]),
                           Width=c(zSemiBoot[,3],zParBoot[,3],zPolyBoot[,3]),
                           which=rep(c("Semiparametric","Correct Model","Polynomial Background Fit"),each=as.numeric(input$nBoot))) 
              plot1 <- ggplot(dta,aes(x=Mix)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "dark green", alpha = 0.2)+
                    xlab("Number of Signal Events")
              plot2 <- ggplot(dta,aes(x=Location)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "dark green", alpha = 0.2)
              plot3 <- ggplot(dta,aes(x=Width)) + 
                    geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2) +
                    geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "dark green", alpha = 0.2)                            
                         
          }
          
          if(input$fitWhat=="Number of Signal events only") print(plot1)
          if(input$fitWhat=="Number of Signal events and Signal Location") {
                  pushViewport(viewport(layout = grid.layout(1, 2)))
                  print(plot1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
                  print(plot2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
          }
          if(input$fitWhat=="All three") {
                  pushViewport(viewport(layout = grid.layout(2, 2)))
                  print(plot1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
                  print(plot2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
                  print(plot3, vp = viewport(layout.pos.row = 2, layout.pos.col = 1))
          }  
   })
   
   output$boottext <- renderText({
         which <- c(T,F,F)
         if(input$whichBoot=="All methods") which<-c(T,T,T)
         if(input$whichData!="Included Background Models") which[2]<-F
         
         if(which[1]) {
            zSemi <- semiFit()$par 
            zSemiBoot <- bootstrap()[[1]]
            sdSemiBoot <- apply(zSemiBoot,2,sd)
         }   
         if(which[2]) {
              zPar <- parFit()$par
              zParBoot <- bootstrap()[[2]]
              sdParBoot <- apply(zParBoot,2,sd) 
         }          
         if(which[3]) {
              zPoly <- polyFit()$par
              zPolyBoot <- bootstrap()[[3]]
              sdPolyBoot <- apply(zPolyBoot,2,sd) 
         } 

         a <- "<table>" 
         a[2] <- paste("<tr><td>&nbsp;</td><th>",red,"Semiparametric Method</th>")
         if(which[2])
              a[2] <- paste(a[2],"<th>",blue,space,"Correct Background Model</th>")
         if(which[3])
              a[2] <- paste(a[2],"<th>",green,space,"Polynomial Background Fit</th>")
         a[2] <- paste(a[2],"</tr>")              
         
         a[3] <- paste("<tr><td>Estimated number of Signal Events</td><td>",space,space,round(zSemi[1,1],1),"&plusmn;",
                 round(zSemi[1,2],1),"</td>")     
         if(which[2])           
              a[3] <- paste(a[3],"<td>",space,space,space,round(zPar[1,1],1),"&plusmn;",round(zPar[1,2],1),"</td>")                                     
         if(which[3])           
              a[3] <- paste(a[3],"<td>",space,space,space,round(zPoly[1,1],1),"&plusmn;",round(zPoly[1,2],1),"</td>")                                     
         a[3] <- paste(a[3],"</tr>") 
         
         a[4] <- paste("<tr><td>Bootstrap 95% CI (Normal Theory)</td><td>",space,space,"(",
                        round(zSemi[1,1]-1.96*sdSemiBoot[1],1),",", round(zSemi[1,1]+1.96*sdSemiBoot[1],1),")</td>")
         if(which[2])           
               a[4] <- paste(a[4],"<td>",space,space,"(",
                            round(zPar[1,1]-1.96*sdParBoot[1],1),",",round(zPar[1,1]+1.96*sdParBoot[1],1),")</td>")  
         if(which[3])           
               a[4] <- paste(a[4],"<td>",space,space,"(",
                            round(zPoly[1,1]-1.96*sdPolyBoot[1],1),",",round(zPoly[1,1]+1.96*sdPolyBoot[1],1),")</td>")  
         a[4] <- paste(a[4],"</tr>")        
         
         a[5] <- paste("<tr><td>Bootstrap 95% CI (Percentile)</td><td>",space,space,"(",
                            round(quantile(zSemiBoot[,1],0.025),1),",",round(quantile(zSemiBoot[,1],0.975),1),")</td>")
         if(which[2])           
               a[5] <- paste(a[5],"<td>",space,space,"(",
                            round(quantile(zParBoot[,1],0.025),1),",",round(quantile(zParBoot[,1],0.975),1),")</td>") 
         if(which[3])           
               a[5] <- paste(a[5],"<td>",space,space,"(",
                            round(quantile(zPolyBoot[,1],0.025),1),",",round(quantile(zPolyBoot[,1],0.975),1),")</td>")          
         a[5] <- paste(a[5],"</tr>")                                                                        
   
         if(input$fitWhat=="Number of Signal events only") return(c(a,"</table>"))

         a[6] <- paste("<tr><td>Estimated Signal Location",space,"</td><td>",space,space,round(zSemi[2,1],3),"&plusmn;",
                 round(zSemi[2,2],3),"</td>")
         if(which[2])                 
               a[6] <- paste(a[6],"<td>",space,space,space,round(zPar[2,1],3),"&plusmn;",round(zPar[2,2],3),"</td>") 
         if(which[3])                 
               a[6] <- paste(a[6],"<td>",space,space,space,round(zPoly[2,1],3),"&plusmn;",round(zPoly[2,2],3),"</td>") 
         a[6] <- paste(a[6],"</tr>")
         a[7] <- paste("<tr><td>Bootstrap 95% CI (Normal Theory)</td><td>",space,space,"(",
                            round(zSemi[2,1]-1.96*sdSemiBoot[2],4),",", 
                            round(zSemi[2,1]+1.96*sdSemiBoot[2],4),")</td>")
         if(which[2])                 
               a[7] <- paste(a[7],"<td>",space,space,"(",
                            round(zPar[2,1]-1.96*sdParBoot[2],4),",",
                            round(zPar[2,1]+1.96*sdParBoot[2],4),")</td>")
         if(which[3])                 
               a[7] <- paste(a[7],"<td>",space,space,"(",
                            round(zPoly[2,1]-1.96*sdPolyBoot[2],4),",",
                            round(zPoly[2,1]+1.96*sdPolyBoot[2],4),")</td>")
         a[7] <- paste(a[7],"</tr>")                     
         a[8] <- paste("<tr><td>Bootstrap 95% CI (Percentile)</td><td>",space,space,"(",
                            round(quantile(zSemiBoot[,2],0.025),4),",",
                            round(quantile(zSemiBoot[,2],0.975),4),")</td>")
         if(which[2])                 
               a[8] <- paste(a[8],"<td>",space,space,"(",
                            round(quantile(zParBoot[,2],0.025),4),",",
                            round(quantile(zParBoot[,2],0.975),4),")</td>")                                                                      
         if(which[3])                 
               a[8] <- paste(a[8],"<td>",space,space,"(",
                            round(quantile(zPolyBoot[,2],0.025),4),",",
                            round(quantile(zPolyBoot[,2],0.975),4),")</td>")                                                                      
         a[8] <- paste(a[8],"</tr>")                            
         if(input$fitWhat=="Number of Signal events and Signal Location") return(c(a,"</table>"))        

         a[9] <- paste("<tr><td>Estimated Signal Width",space,"</td><td>",space,space,round(zSemi[3,1],4),"&plusmn;",
                 round(zSemi[3,2],4),"</td>")
         if(which[2])                 
               a[9] <- paste(a[9],"<td>",space,space,space,round(zPar[3,1],4),"&plusmn;",round(zPar[3,2],4),"</td>")      
         if(which[3])                 
               a[9] <- paste(a[9],"<td>",space,space,space,round(zPoly[3,1],4),"&plusmn;",round(zPoly[3,2],4),"</td>")
         a[9] <- paste(a[9],"</tr>")
         a[10] <-  paste("<tr><td>Bootstrap 95% CI (Normal Theory)</td><td>",space,space,"(",
                            round(zSemi[3,1]-1.96*sdSemiBoot[3],4),",", 
                            round(zSemi[3,1]+1.96*sdSemiBoot[3],4),")</td>")
         if(which[2])                 
               a[10] <- paste(a[10],"<td>",space,space,"(",
                            round(zPar[3,1]-1.96*sdParBoot[3],4),",",
                            round(zPar[3,1]+1.96*sdParBoot[3],4),")</td>") 
         if(which[3])                 
               a[10] <- paste(a[10],"<td>",space,space,"(",
                            round(zPoly[3,1]-1.96*sdPolyBoot[3],4),",",
                            round(zPoly[3,1]+1.96*sdPolyBoot[3],4),")</td>")
         a[10] <- paste(a[10],"</tr>")                   
         a[11] <- paste("<tr><td>Bootstrap 95% CI (Percentile)</td><td>",space,space,"(",
                            round(quantile(zSemiBoot[,3],0.025),4),",",
                            round(quantile(zSemiBoot[,3],0.975),4),")</td>")
         if(which[2])                 
               a[11] <- paste(a[11],"<td>",space,space,"(",
                            round(quantile(zParBoot[,3],0.025),4),",",
                            round(quantile(zParBoot[,3],0.975),4),")</td>")   
         if(which[3])                 
               a[11] <- paste(a[11],"<td>",space,space,"(",
                            round(quantile(zPolyBoot[,3],0.025),4),",",
                            round(quantile(zPolyBoot[,3],0.975),4),")</td>")
         a[11] <- paste(a[11],"</tr>")                                                                            

        c(a,"</table>")    
   })           

    simulate <- eventReactive(input$button, {
        A <- as.numeric(input$A)
        B <- as.numeric(input$B)
            
        which <- c(T,F,F)
        if(input$whichSim=="All methods") which<-c(T,T,T)
        if(input$whichData=="Enter your own background function") 
              Model <- "Other"
        else       
              Model <- input$Model    

        n <- as.numeric(input$nB)+as.numeric(input$nS)
        alpha <- as.numeric(input$nS)/n
        progress <- shiny::Progress$new(session, min=1, max=15)
        on.exit(progress$close())
        progress$set(value = 0,message="0 % done") 
        zSemiSim <- matrix(0,as.numeric(input$nSim),6)            
        zParSim <- matrix(0,as.numeric(input$nSim),6)                    
        zPolySim <- matrix(0,as.numeric(input$nSim),6)                    
        for(i in 1:as.numeric(input$nSim)) {
             if(input$whichData=="Enter your own background function") {
                    z <- genDataOwn(input$nB,input$nS,input$nBy,input$mu,input$sig,A=A,B=B,input$backfun)
                    x <- z$x
                    y <- z$y
             }
             else { 
                if(input$Model=="Exponential") {
                    x<-semiData(as.numeric(input$nB),as.numeric(input$nS),"Exponential",c(as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$lambda)),input$A,input$B)
                    y<-semiData(as.numeric(input$nBy),0,"Exponential",c(as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$lambda)),input$A,input$B)
                }        
                if(input$Model=="Flat") {
                    x<-semiData(as.numeric(input$nB),as.numeric(input$nS),"Flat",c(as.numeric(input$mu),as.numeric(input$sig)),A,B)
                    y<-semiData(as.numeric(input$nBy),0,"Flat",c(as.numeric(input$mu),as.numeric(input$sig)),A,B)
                }    
                if(input$Model=="Beta") {
                    x<-semiData(as.numeric(input$nB),as.numeric(input$nS),"Beta",c(as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$pbeta1),as.numeric(input$pbeta2)))    
                    y<-semiData(as.numeric(input$nBy),0,"Beta",c(as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$pbeta1),as.numeric(input$pbeta2)))    
                }
             } 
             if(which[1]) 
                  zSemiSim[i,] <- c(sp2EM.fit(x,y,p=c(alpha,as.numeric(input$mu),as.numeric(input$sig)),fitWhat="Mix",
                                  A=A,B=B)$par)
                               
             if(which[2]) {
                if(input$whichData=="Enter your own background function")     
                      p <- c(alpha,as.numeric(input$mu),as.numeric(input$sig))
                else {      
                    if(input$Model=="Exponential") 
                      p <- c(alpha,as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$lambda))
                    if(input$Model=="Flat") 
                      p <- c(alpha,as.numeric(input$mu),as.numeric(input$sig))
                    if(input$Model=="Beta") 
                      p <- c(alpha,as.numeric(input$mu),as.numeric(input$sig),as.numeric(input$pbeta1),as.numeric(input$pbeta2))
                }      
                zParSim[i,]<- c(par2EM.fit(x,y,p=p,fitWhat="Mix",Model=Model,dBack=input$backfun,A=A,B=B)$par[1:3, ])
             }                   
             if(which[3]) 
                  zPolySim[i,] <- c(poly2EM.fit(x,y,p=c(alpha,as.numeric(input$mu),as.numeric(input$sig)),fitWhat="Mix",A=A,B=B)$par)             
             
             if( (i/as.numeric(input$nSim)*100)%%10==0) 
                    progress$set(value = i/as.numeric(input$nSim),
                                message=paste(i/as.numeric(input$nSim)*100,"% done",sep=""))      
        }
        return(list(zSemiSim,zParSim,zPolySim))

   }) 
   output$simplot <- renderPlot({
          if(input$whichData=="Upload your own") return(NULL)
          which <- c(T,F,F)
          if(input$whichSim=="All methods") which<-c(T,T,T)
          zSemiSim <- simulate()[[1]]
          zParSim <- simulate()[[2]]
          zPolySim <- simulate()[[3]]
          dta <- data.frame(Mix=c(zSemiSim[,1],zParSim[,1],zPolySim[,1]),
                           Location=c(zSemiSim[,2],zParSim[,2],zPolySim[,2]),
                           Width=c(zSemiSim[,3],zParSim[,3],zPolySim[,3]),
                           which=rep(c("Semiparametric","Correct Model","Polynomial Background Fit"),each=as.numeric(input$nSim))) 
          plot1 <- ggplot(dta,aes(x=Mix))+xlab("Number of Signal Events") 
          if(which[1])
                plot1 <- plot1 + geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) 
          if(which[2])
                plot1 <- plot1 + geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2)
          if(which[3])
                plot1 <- plot1 + geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "green", alpha = 0.2)                    

          plot2 <- ggplot(dta,aes(x=Location)) + xlab("Signal Location") 
          if(which[1])
                plot2 <- plot2 + geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) 
          if(which[2])
                plot2 <- plot2 + geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2)
          if(which[3])
                plot2 <- plot2 + geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "green", alpha = 0.2)                    

          plot3 <- ggplot(dta,aes(x=Width)) + xlab("Signal Width")
          if(which[1])
                plot3 <- plot3 + geom_histogram(data=subset(dta,which == 'Semiparametric'),fill = "red", alpha = 0.2) 
          if(which[2])
                plot3 <- plot3 + geom_histogram(data=subset(dta,which == 'Correct Model'),fill = "blue", alpha = 0.2)
          if(which[3])
                plot3 <- plot3 + geom_histogram(data=subset(dta,which == 'Polynomial Background Fit'),fill = "green", alpha = 0.2)                    

          if(input$fitWhat=="Number of Signal events only") print(plot1)
          if(input$fitWhat=="Number of Signal events and Signal Location") {
                  pushViewport(viewport(layout = grid.layout(1, 2)))
                  print(plot1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
                  print(plot2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
          }
          if(input$fitWhat=="All three") {
                  pushViewport(viewport(layout = grid.layout(2, 2)))
                  print(plot1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
                  print(plot2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
                  print(plot3, vp = viewport(layout.pos.row = 2, layout.pos.col = 1))
          }  
   })
   
   output$simtext <- renderText({
         if(input$whichData=="Upload your own")  
                      return("<h3>Not possible for uploaded data!</h3>")
         which <- c(T,F,F)
         if(input$whichSim=="All methods") which<-c(T,T,T)         
         zSemiSim <- simulate()[[1]]
         zParSim <- simulate()[[2]]
         zPolySim <- simulate()[[3]]

         a <- "<table width=\"500\">" 
         a[2] <- paste("<tr><td>&nbsp;</td><th>",red,"Semiparametric Method</th>")
         if(which[2]) a[2] <-paste(a[2],"<th>", blue,"Correct Background Model</th>")
         if(which[3]) a[2] <-paste(a[2],"<th>", green,"Polynomial Background Fit</th>")
         a[2] <- paste(a[2],"</tr>")
         
         low <- zSemiSim[,1]-1.96*zSemiSim[,4]
         high <- zSemiSim[,1]+1.96*zSemiSim[,4]
         cov <- paste(round((as.numeric(input$nSim)-sum(low>as.numeric(input$nS))-sum(high<as.numeric(input$nS)))/as.numeric(input$nSim)*100,1),"%",sep="")
         a[3] <- paste("<tr><td>Number of signal events:</td><td>",cov,"</td>")

         if(which[2]) {
            low <- zParSim[,1]-1.96*zParSim[,4]
            high <- zParSim[,1]+1.96*zParSim[,4]
            cov <- paste(round((as.numeric(input$nSim)-sum(low>as.numeric(input$nS))-sum(high<as.numeric(input$nS)))/as.numeric(input$nSim)*100,1),"%",sep="")
            a[3] <-paste(a[3],"<td>",cov,"</td>")  
         }
         if(which[3]) {     
            low <- zPolySim[,1]-1.96*zPolySim[,4]
            high <- zPolySim[,1]+1.96*zPolySim[,4]
            cov <- paste(round((as.numeric(input$nSim)-sum(low>as.numeric(input$nS))-sum(high<as.numeric(input$nS)))/as.numeric(input$nSim)*100,1),"%",sep="")
            a[3] <-paste(a[3],"<td>",cov,"</td>")
         }
         a[3] <- paste(a[3],"</tr>")   

         c("<h5>Nominal Coverage: 95%</h5>",a,"</table>")    
   })   
   output$simtext1 <- renderText({           
        "<h5>Coverage study is currently possible only for number of signal events because fitting for additional
        parameters fails to converge on occasion</h5>"
          
   })     

   bandwidth <- reactive({
         m <- 10
         A <- as.numeric(input$A)
         B <- as.numeric(input$B) 
         bw1 <- as.numeric(input$bw1)
         bw2 <- as.numeric(input$bw2)
         if(input$fitWhat=="Number of Signal events only") fitWhat<-"Mix"
         if(input$fitWhat=="Number of Signal events and Signal Location") fitWhat<-"MixLoc"
         if(input$fitWhat=="All three") fitWhat<-"All"
         p <- c(as.numeric(input$nS)/(as.numeric(input$nB)+as.numeric(input$nS)),
                        as.numeric(input$mu),as.numeric(input$sig))
         semiData <- sp2EM.fit(data()$x,data()$y,p=p,fitWhat=fitWhat,A=A,B=B)
         Ty <- semiData$Ty
         if(bw1>0) {
                bw <- seq(bw1,bw2,length=m)
                names(bw) <- 1:m
         }       
         else {
            bw <- rep(0,5) 
            names(bw) <- c("nrd","nrd0","ucv","bcv","SJ")        
            m <- 5
            bw[1] <- bw.nrd(Ty)
            bw[2] <- bw.nrd0(Ty)
            bw[3] <- bw.ucv(Ty)
            bw[4] <-  bw.bcv(Ty)
            bw[5] <- bw.SJ(Ty)      
         }   
         out <- as.list(0:m)
         out[[1]] <- bw
         for(i in 1:m) 
             out[[i+1]] <- sp2EM.fit(data()$x,data()$y,p=p,fitWhat=fitWhat,A=A,B=B,bw=bw[i])
         out
   })
   
   output$bandtext <- renderText({ 
         out <- bandwidth()
         m <- length(out)-1
         bw <- out[[1]]
         O <- order(bw)+1
         if(names(bw)[1]=="nrd") 
              a <- "<tr><th>Bandwidth Selection Method</th><th>Bandwidth</th><th>Number of Signal Events</th>"
         else 
            a <- "<tr><th>Bandwidth</th><th>Number of Signal Events</th>"                 
         if(input$fitWhat!="Number of Signal events only") a <- paste(a,"<th>Signal Location</th>")
         if(input$fitWhat=="All three") a <- paste(a,"<th>Signal Width</th>")
         a <- paste(a,"</tr>")
         for(i in 1:m) {
              a[i+1] <- "<tr>"
              if(names(bw)[1]=="nrd") a[i+1] <- paste(a[i+1],"<td>",names(bw)[O[i]-1],"</td>")  
              a[i+1] <- paste(a[i+1],"<td>",round(out[[O[i]]]$bw,4),"</td><td>",
                              round(out[[O[i]]]$par[1,1],1),"&plusmn;",round(out[[O[i]]]$par[1,2],1),"</td>")
              if(input$fitWhat!="Number of Signal events only") 
                  a[i+1] <- paste(a[i+1],"<td>",round(out[[O[i]]]$par[2,1],4),"&plusmn;",round(out[[O[i]]]$par[2,2],4),"</td>")
              if(input$fitWhat=="All three")
                  a[i+1] <- paste(a[i+1],"<td>",round(out[[O[i]]]$par[3,1],4),"&plusmn;",round(out[[O[i]]]$par[3,2],4),"</td>")
              a[i+1] <- paste(a[i+1],"</tr>")
         }
         c("<table width=\"500\">",a,"</table>")
          
   })
  
   output$bandplot <- renderPlot({  
        out <- bandwidth()
        m <- length(out)
        if(names(out[[1]])[1]!="nrd") {
            X <- data.frame(x=rep(out[[2]]$x,2),
                            y=c(out[[2]]$xf,out[[m]]$xf),
                            Method = rep(c("Lowest Bandwidth","Highest Bandwidth"),each=length(out[[2]]$xf)))
            TX <- data.frame(x=rep(out[[2]]$Tx,2),
                            y=c(out[[2]]$Txf,out[[m]]$Txf),
                            Method = rep(c("Lowest Bandwidth","Highest Bandwidth"),each=length(out[[2]]$xf)))                
        }
        else {
            X <- data.frame(x=rep(out[[2]]$x,5),
                            y=c(out[[2]]$xf,out[[3]]$xf,out[[4]]$xf,out[[5]]$xf,out[[6]]$xf),
                            Method = rep(names(out[[1]]),each=length(out[[2]]$xf)))
            TX <- data.frame(x=rep(out[[2]]$Tx,5),
                            y=c(out[[2]]$Txf,out[[3]]$Txf,out[[4]]$Txf,out[[5]]$Txf,out[[6]]$Txf),
                            Method = rep(names(out[[1]]),each=length(out[[2]]$xf)))
      }                                   
      bwHist <- diff(range(X[,1]))/100
      plt1 <- ggplot(data=X,aes(x,colour=Method))+
             geom_histogram(aes(y = ..density..),color="black",fill="white", binwidth = bwHist)+
             geom_line(aes(x,y))+
             theme(legend.position = "none")+
             xlab("Original Data")+ylab("")
                          
      bwHist <- diff(range(TX[,1]))/100
      plt2 <- ggplot(data=TX,aes(x,colour=Method))+
            geom_histogram(aes(y = ..density..),color="black",fill="white", binwidth = bwHist)+
            geom_line(aes(x,y))+
            theme(legend.position="bottom") +
            xlab("Transformed Data")+ylab("")                        
      
      pushViewport(viewport(layout = grid.layout(1, 2)))
      print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
      print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
   })
   
   output$bandtext1 <- renderText({ 
          if(names(bandwidth()[[1]])[1]!="nrd") return("")
          a <- "<h5>here we use 5 different bandwidth selection methods:</h5>"
          a[2] <- "<h5>nrd = default rule for Gaussian distribution</h5>"
          a[3] <- "<h5>nrd0 = Scott's version of nrd</h5>"
          a[4] <- "<h5>SJ = Sheather&Jones </h5>"
          a[5] <- "<h5>ucv = unbiased cross-valiadation</h5>"
          a[6] <- "<h5>bcv = biased cross valiadation</h5>"
          a[7] <- "<h5>or the user can set a range of bandwidths</h5>"
          a
   })
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/10/2015<br>Last updated: 12/6/2015</h6>")
      
  })       


})

