fitBernstein <-
function (x,A=0,B=1) 
{
      x <- sort((x-A)/(B-A))
      n <- length(x)
      calcBernstein <- function(x,m) {
            A <- matrix(0,length(x),m+1)
             k <- 0:m
            coefs <- factorial(m+1)/factorial(k)/factorial(m-k)
            for(k in 0:m) 
               A[,k+1] <- coefs[k+1]*x^k*(1-x)^(m-k)
            A
      }         
      EM <- function(m) {
          A <- calcBernstein(x,m)     
          p <- rep(1,m+1)/(m+1)
          p1 <- p
          repeat {
              y <- p[1]*A[,1]
              for(k in 2:(m+1)) y <- y+p[k]*A[,k]
              for(k in 1:(m+1)) p1[k] <- mean(p[k]*A[,k]/y)
              p1 <- p1/sum(p1)

              if(sum(abs(p-p1))<0.0001) break
              p <- p1
         } 
         list(par=p,y=y,loglike=sum(log(y)))
      }
      ll <- c(0,0,0)
      ll[2] <- EM(1)$loglike
      ll[3] <- EM(2)$loglike
      if(ll[3]-ll[1]<1.35 & ll[2]-ll[1]<1.35) 
          return(list(par=c(1/B-A),y=rep(1/(B-A),n),loglike=(-n)*log(B-A)))

      m<-2        
      repeat {
          m <- m+1
          z <- EM(m)
          ll <- c(ll,z$loglike)
          if(ll[m+1]-ll[m-1]<1.35 & ll[m+1]-ll[m]<1.35) 
              break
      } 
      z <- EM(m-1)
      z$y <- z$y/(B-A)
      z   
}
poly2EM.fit <-
function (x,y,p,fitWhat="All",A=0,B=1,likeOut=F) 
{
      require(numDeriv)
      dSig <- function(x,p) dnorm(x,p[1],p[2])/diff(pnorm(c(A,B),p[1],p[2]))      
      x <- sort(x)
      y <- sort(y)
      zx <- trw(x,A=A,B=B)
      zy <- trw(y,A=A,B=B)
      z <- fitBernstein(y,A=A,B=B)
      yf <- z$y
      deg <- length(z$par)-1
      xfB <- approx(y,yf,xout=x,rule=2)$y
      par <- EM(x,xfB,p=p,fitWhat=fitWhat,A=A,B=B)

      if(fitWhat=="All") 
          log.ll <- function(x,xData,pfix) {
               yhat <- (1-x[1])*xfB+x[1]*dSig(xData,x[2:3])
               yhat[yhat<=0] <- min(yhat[yhat>0])    
              -sum(log(yhat))
          }    
      if(fitWhat=="Mix") {
          yS <- dSig(x,p[2:3])
          log.ll <- function(x,xData,pfix) { 
               yhat <- (1-x[1])*xfB+x[1]*yS
               yhat[yhat<=0] <- min(yhat[yhat>0])  
              -sum(log(yhat))
          }           
      }    
      if(fitWhat=="MixLoc") 
          log.ll <- function(x,xData,pfix)  {
               yhat <- (1-x[1])*xfB+x[1]*dSig(xData,c(x[2],pfix[3]))
               yhat[yhat<=0] <- min(yhat[yhat>0])    
              -sum(log(yhat))
          }        
      if(likeOut) return(log.ll(p,x,p))
      if(fitWhat=="Mix") {
            hess <- hessian(log.ll,x=par[1],xData=x,pfix=par)
            err <- 1/sqrt(hess)
      }      
      else {    
          if(fitWhat=="MixLoc") hess <- hessian(log.ll,x=par[1:2],xData=x,pfix=par)
          else hess <- hessian(log.ll,x=par,xData=x,pfix=par)
          err <- sqrt(diag(solve(hess)))
      }    

      alpha <- length(x)*c(par[1],err[1])
      loc <- c(p[2],0)
      if(fitWhat=="MixLoc" | fitWhat=="All") 
          loc <-  c(par[2],err[2])
      width <- c(p[3],0)   
      if(fitWhat=="All") 
          width <-  c(par[3],err[3])
      par <- rbind(alpha,loc,width)  
      xf <- (1-alpha[1]/length(x))*xfB+alpha[1]/length(x)*dSig(x,par[2:3,1])
      list(x=x,xf=xf,Tx=zx$Tx,Txf=xf*zx$dxdy,y=y,yf=yf,Ty=zy$Tx,Tyf=yf*zy$dxdy,par=par,deg=deg)
}
par2EM.fit <-
function (x,y,p,fitWhat="All",Model,dBack,A=0,B=1) 
{
      dSig <- function(x,p) dnorm(x,p[1],p[2])/diff(pnorm(c(A,B),p[1],p[2]))
      x <- sort(x)
      y <- sort(y)
      zx <- trw(x,A=A,B=B)
      zy <- trw(y,A=A,B=B)
  
      if(Model=="Other") {
          eval(parse(text = paste("mydf <- function(x)",dBack,sep="")))
          I <- integrate(mydf,A,B)$value
          xfB <- mydf(x)/I
          yf <- mydf(y)/I
      }    
      if(Model=="Flat") {
          xfB <- rep(1/(B-A),length(x))
          yf <- rep(1/(B-A),length(y))
      }  
      if(Model=="Exponential") {
            backPar <- backmle(y,c(A,B,1,p[4]))[1]
            yf <- dexp(y,backPar)/diff(pexp(c(A,B),backPar))
            xfB <- dexp(x,backPar)/diff(pexp(c(A,B),backPar))
      }
      if(Model=="Beta") {
            backPar <- backmle(y,c(A,B,2,p[4:5]))[1:2]
            yf <- dbeta(y,backPar[1],backPar[2])
            xfB <- dbeta(x,backPar[1],backPar[2])
      }     
      
      par <- EM(x,xfB,p=p,fitWhat=fitWhat,A=A,B=B)    
      if(fitWhat=="All") 
          log.ll <- function(x,xData,pfix) {
               yhat <- (1-x[1])*xfB+x[1]*dSig(xData,x[2:3])
               yhat[yhat<=0] <- min(yhat[yhat>0])    
              -sum(log(yhat))
          }    
      if(fitWhat=="Mix") {
          yS <- dSig(x,p[2:3])
          log.ll <- function(x,xData,pfix) { 
               yhat <- (1-x[1])*xfB+x[1]*yS
               yhat[yhat<=0] <- min(yhat[yhat>0])  
              -sum(log(yhat))
          }           
      }    
      if(fitWhat=="MixLoc") 
          log.ll <- function(x,xData,pfix)  {
               yhat <- (1-x[1])*xfB+x[1]*dSig(xData,c(x[2],pfix[3]))
               yhat[yhat<=0] <- min(yhat[yhat>0])    
              -sum(log(yhat))
          }        
      
      if(fitWhat=="Mix") {
            hess <- hessian(log.ll,x=par[1],xData=x,pfix=par[1:3])
            err <- 1/sqrt(hess)
      }      
      else {    
          if(fitWhat=="MixLoc") hess <- hessian(log.ll,x=par[1:2],xData=x,pfix=par[1:3])
          else hess <- hessian(log.ll,x=par[1:3],xData=x,pfix=par[1:3])
          err <- sqrt(diag(solve(hess)))
      }
      alpha <- length(x)*c(par[1],err[1])
      loc <- c(p[2],0) 
      if(fitWhat=="MixLoc" | fitWhat=="All") 
          loc <-  c(par[2],err[2])
      width <- c(p[3],0)   
      if(fitWhat=="All") 
          width <-  c(par[3],err[3])    
      if(length(p)==3) par <- rbind(alpha,loc,width)    
      else {
          backPar <- cbind(backPar,0)
          par <- rbind(alpha,loc,width,backPar)    
          rownames(par)[4:length(p)] <- paste("backPar",1:(length(p)-3),sep="")
          colnames(par)<- c("Estimate","Error")
      }   
       
      xf <- (1-par[1,1]/length(x))*xfB+par[1,1]/length(x)*dSig(x,par[2:3,1])
      
      list(x=x,xf=xf,Tx=zx$Tx,Txf=xf*zx$dxdy,y=y,yf=yf,Ty=zy$Tx,Tyf=yf*zy$dxdy,par=par)
}
sp2EM.fit <-
function (x,y,p,fitWhat="All",A=0,B=1,likeOut=F, bw) 
{ 
      x <- sort(x)
      y <- sort(y)

      z <- trw(x, A, B, 2)
      Tx <- z$Tx
      dxdy.x <- z$dxdy
      z <- trw(y, A, B, 2)
      Ty <- z$Tx
      dxdy.y <- z$dxdy
      
      if(missing(bw)) bw <- bw.nrd(Ty)
      g <- density(Ty,bw=bw,n=1024)
      Tyf <- approx(g$x,g$y,Ty)$y
      
      TxfB <- approx(g$x,g$y,Tx)$y
      TxfB <- TxfB[order(Tx)]
      Tx <- sort(Tx)
      TxfB[is.na(TxfB)] <- 0
      TxfB <- TxfB/sum( (TxfB[-1]+TxfB[-length(TxfB)])/2*(Tx[-1]-Tx[-length(TxfB)]) )
      
      dSig <- function(x,p) dnorm(x,p[1],p[2])/diff(pnorm(c(A,B),p[1],p[2]))*dxdy.x
      par <- EM(x,TxfB,p=p,fitWhat=fitWhat,A=A,B=B,dxdy=dxdy.x)

      if(fitWhat=="All") 
          log.ll <- function(x,xData,pfix) {
               yhat <- (1-x[1])*TxfB+x[1]*dSig(xData,x[2:3])
               yhat[yhat<=0] <- min(yhat[yhat>0])    
              -sum(log(yhat))
          }    
      if(fitWhat=="Mix") {
          yS <- dSig(x,p[2:3])
          log.ll <- function(x,xData,pfix) { 
               yhat <- (1-x[1])*TxfB+x[1]*yS
               yhat[yhat<=0] <- min(yhat[yhat>0])  
              -sum(log(yhat))
          }           
      }    
      if(fitWhat=="MixLoc") 
          log.ll <- function(x,xData,pfix)  {
               yhat <- (1-x[1])*TxfB+x[1]*dSig(xData,c(x[2],pfix[3]))
               yhat[yhat<=0] <- min(yhat[yhat>0])    
              -sum(log(yhat))
          }        

      if(fitWhat=="Mix") {
            hess <- hessian(log.ll,x=par[1],xData=x,pfix=par)
            err <- 1/sqrt(hess)
      }      
      else {    
          if(fitWhat=="MixLoc") hess <- hessian(log.ll,x=par[1:2],xData=x,pfix=par)
          else hess <- hessian(log.ll,x=par,xData=x,pfix=par)
          err <- sqrt(diag(solve(hess)))
      }    

      alpha <- length(x)*c(par[1],err[1])
      loc <- c(p[2],0)
      if(fitWhat=="MixLoc" | fitWhat=="All") 
          loc <-  c(par[2],err[2])
      width <- c(p[3],0)   
      if(fitWhat=="All") 
          width <-  c(par[3],err[3])
      par <- rbind(alpha,loc,width)  
      Txf <- (1-alpha[1]/length(x))*TxfB+alpha[1]/length(x)*dSig(x,par[2:3,1])
      list(x=x,xf=Txf*(B-A)/(x-A)/(B-x),Tx=Tx,Txf=Txf,y=y,yf=Tyf*(B-A)/(y-A)/(B-y),Ty=Ty,Tyf=Tyf,par=par,bw=bw)
}
EM <-
function(x,yB,par,fitWhat,A=0,B=1,dxdy) {
            if(missing(dxdy)) dxdy=rep(1,length(x))
            oldpar <- par
            if(fitWhat=="Mix")
                  yS <- dnorm(x,oldpar[2],oldpar[3])/diff(pnorm(c(A,B),oldpar[2],oldpar[3]))*dxdy
            repeat {
                  if(fitWhat!="Mix")
                      yS <- dnorm(x,oldpar[2],oldpar[3])/diff(pnorm(c(A,B),oldpar[2],oldpar[3]))*dxdy  
                  y <- (1-oldpar[1])*yB+oldpar[1]*yS
                  w <- oldpar[1]*yS/y
                  par[1] <- mean(w,na.rm=T)
                  if(fitWhat=="MixLoc" | fitWhat=="All") {
                        par[2] <- sum(x*w,na.rm=T)/sum(w,na.rm=T)
                        if(fitWhat=="All") par[3] <- sqrt(sum(w*(x-par[2])^2,na.rm=T)/sum(w,na.rm=T))
                  }
                  if(sum(abs(par-oldpar))<1/1e7) break
                  oldpar <- par
           }
           par 
}
gof <-
function (x,y,nbins,fitWhat="Mix",Model,pStart,A=0,B=1,nMC=100,which=c(T,T,T)) 
{
    if(Model=="None") which[2]<-F
    simpsonRule <- function (bins,x,f) {
        midp <- (bins[-1]+bins[-length(bins)])/2
        fbins <- approx(x,f,bins,rule=2)$y
        fmid <- approx(x,f,midp,rule=2)$y
        h <- diff(bins)
        yp <- (fbins[-1]+4*fmid+fbins[-length(fbins)])
        yp/6*h
    }
    FSemi <- function(x,y) {
        c(0,cumsum((y[-1]+y[-nx])*diff(x))/2)
    }
    x <- sort(x)
    nx <- length(x)    
    if(missing(y)) {
         z <- sp.fit(x=x,p=pStart[1:3],fitWhat=fitWhat,A=A,B=B)
         Tx <- z$Tx
         ySemi <- z$Txf 
         if(which[2])
            pPar <- par.fit(x=x,p=pStart,fitWhat=fitWhat,Model=Model,A=A,B=B)$par[,1]
   }    
   else {
         y <- sort(y)
         ny <- length(y)
         z <- sp2EM.fit(x=x,y=y,p=pStart[1:3],fitWhat=fitWhat,A=A,B=B)
         pSemi <- z$par[1:3,1]
         Tx <- z$Tx
         ySemi <- z$Txf
         if(which[3]) {
            z <- poly2EM.fit(x=x,y=y,p=pStart[1:3],fitWhat=fitWhat,A=A,B=B)
            pPoly <- z$par[1:3,1]
            yPoly <- z$xf
            ndPoly <- z$deg
         }   
         if(which[2])
              pPar <- par2EM.fit(x=x,y=y,p=pStart,fitWhat=fitWhat,Model=Model,A=A,B=B)$par[,1]
    }   
    bins <- seq(A,B,length=nbins+1)
    Tbins <- c(min(Tx)-1/100,trw(bins[2:nbins],A=A,B=B)$Tx,max(Tx)+1/100)
    O <- hist(x,breaks=bins,plot=FALSE)$counts

    if(fitWhat=="All") np <- 3
    if(fitWhat=="MixLoc") np <- 2
    if(fitWhat=="Mix") np <- 1
    if(Model=="Exponential") {
        npPar <- np+1 
        FPar <- function(x,p) (1-p[1]/nx)*pexp(x,p[4])/diff(pexp(c(A,B),p[4]))+p[1]/nx*pnorm(x,p[2],p[3])
    }    
    if(Model=="Flat" | Model=="AlmostFlat") {
        npPar <- np
        FPar <- function(x,p) (1-p[1]/nx)*(x-A)/(B-A)+p[1]/nx*pnorm(x,p[2],p[3])
    }    
    if(Model=="Beta") {
        npPar <- np+2 
        FPar <- function(x,p) (1-p[1]/nx)*pbeta(x,p[4],p[5])+p[1]/nx*pnorm(x,p[2],p[3])
    }
    
    if(which[1]) {
        ESemi <- nx*simpsonRule(Tbins,Tx,ySemi)
        chisqSemi <- sum((O-ESemi)^2/ESemi)  
        Fx <- FSemi(Tx,ySemi)
        Fx[Fx<=0] <- min(Fx[Fx>0])
        Fx[Fx>=1] <- max(Fx[Fx<1])
        KSSemi <- max(c(Fx-(0:(nx-1))/nx,(1:nx)/nx-Fx))
        ADSemi <- (-nx-sum( c(2*(1:nx)-1)/nx*(log(Fx)+log(1-Fx[nx:1]))))
    }    
    else {chisqSemi<-NA;KSSemi<-NA;ADSemi<-NA}

    if(which[2]) {
        EPar <- nx*diff(FPar(bins,pPar))
        chisqPar <- sum((O-EPar)^2/EPar) 
        Fx <- FPar(x,pPar)
        KSPar <- max(c(Fx-(0:(nx-1))/nx,(1:nx)/nx-Fx))
        ADPar <- (-nx-sum( c(2*(1:nx)-1)/nx*(log(Fx)+log(1-Fx[nx:1]))))
    }    
    else {chisqPar<-NA;KSPar<-NA;ADPar<-NA}
    
    if(which[3]) {
        EPoly <- nx*simpsonRule(bins,x,yPoly)
        chisqPoly <- sum((O-EPoly)^2/EPoly)  
        Fx <- FSemi(x,yPoly)
        Fx[Fx<=0] <- min(Fx[Fx>0])
        Fx[Fx>=1] <- max(Fx[Fx<1])
        KSPoly <- max(c(Fx-(0:(nx-1))/nx,(1:nx)/nx-Fx))
        ADPoly <- (-nx-sum( c(2*(1:nx)-1)/nx*(log(Fx)+log(1-Fx[nx:1]))))
    }
    else {chisqPoly<-NA;KSPoly<-NA;ADPoly<-NA}
    
    out <- matrix(0,12,3)
    out[,1] <- c(chisqPar,chisqSemi,chisqPar,chisqSemi, KSPar,KSSemi, ADPar,ADSemi, 
                chisqPoly,chisqPoly,KSPoly,ADPoly)
    rownames(out) <- c("chisqPar (LS)","chisqSemi (LS)","chisqPar (MC)","chisqSemi (MC)","KSPar",
                      "KSSemi","ADPar","ADSemi","chisqPoly","chisqPoly","KSPoly","ADPoly")    
    if(which[1])  
        out[2,2:3] <- c(1-pchisq(chisqSemi,nbins-1-np),nbins-1-np)
    if(which[2])  
        out[1,2:3] <- c(1-pchisq(chisqPar,nbins-1-npPar),nbins-1-npPar)
    if(which[3])
        out[9,2:3] <- c(1-pchisq(chisqPoly,nbins-1-np-ndPoly),nbins-1-np-ndPoly)
    colnames(out) <- c("TS","pvalue","df")

    mc <- matrix(0,nMC,9)
    for(i in 1:nMC) {
          x1 <- sample(x,size=nx,replace=T)
          O <- hist(x1,breaks=bins,plot=FALSE)$counts
          if(!missing(y))
              y1 <- sample(y,size=ny,replace=T)
          if(which[1]) {
              if(missing(y)) z <- sp.fit(x=x1,p=pStart[1:3],fitWhat=fitWhat,A=A,B=B)
              else z <- sp2EM.fit(x=x1,y=y1,p=pStart[1:3],fitWhat=fitWhat,A=A,B=B) 
              Tbins[c(1,nbins+1)] <- c(min(z$Tx)-1/100,max(z$Tx)+1/100)
              ESemi <- nx*simpsonRule(Tbins,z$Tx,z$Txf)
              mc[i,2] <- sum((O-ESemi)^2/ESemi)  
              Fx <- FSemi(z$Tx,z$Txf)
              Fx[Fx<=0] <- min(Fx[Fx>0])
              Fx[Fx>=1] <- max(Fx[Fx<1])
              mc[i,4] <- max(c(Fx-(0:(nx-1))/nx,(1:nx)/nx-Fx))
              mc[i,6]<- (-nx-sum( c(2*(1:nx)-1)/nx*(log(Fx)+log(1-Fx[nx:1]))))
        }    
        if(which[2]) {
            if(missing(y)) 
                  pPar <- par.fit(x=x1,p=pStart,fitWhat=fitWhat,Model=Model,A=A,B=B)$par[,1]
            else 
                  pPar <- par2EM.fit(x=x1,y=y1,p=pStart,fitWhat=fitWhat,Model=Model,A=A,B=B)$par[,1]                      
            EPar <- nx*diff(FPar(bins,pPar))
            mc[i,1] <- sum((O-EPar)^2/EPar) 
            Fx <- FPar(x,pPar)
            mc[i,3] <- max(c(Fx-(0:(nx-1))/nx,(1:nx)/nx-Fx))
            mc[i,5] <- (-nx-sum( c(2*(1:nx)-1)/nx*(log(Fx)+log(1-Fx[nx:1]))))
        }            
        if(which[3] & !missing(y)) {
            z <- poly2EM.fit(x=x1,y=y1,p=pStart[1:3],fitWhat=fitWhat,A=A,B=B) 
            EPoly <- nx*simpsonRule(bins,z$x,z$f)
            mc[i,7] <- sum((O-EPoly)^2/EPoly)  
            Fx <- FSemi(z$x,z$xf)
            Fx[Fx<=0] <- min(Fx[Fx>0])
            Fx[Fx>=1] <- max(Fx[Fx<1])
            mc[i,8] <- max(c(Fx-(0:(nx-1))/nx,(1:nx)/nx-Fx))
            mc[i,9] <- (-nx-sum( c(2*(1:nx)-1)/nx*(log(Fx)+log(1-Fx[nx:1]))))
       }
    }     
    for(i in c(2,4,6)) out[i+2,2] <- sum(mc[,i]>out[i+2,1])/nMC    
    if(which[2])
        for(i in c(1,3,5)) out[i+2,2] <- sum(mc[,i]>out[i+2,1])/nMC
    if(which[3])
        for(i in c(7,8,9)) out[i+2,2] <- sum(mc[,i]>out[i+2,1])/nMC            
    out              
}
semiData <-
function (nB,nS,Model,par,A=0,B=1) 
{
       if(Model== "Exponential") 
            rB <- function() rexp(nB,par[3])
       if(Model== "Flat") 
            rB <- function() runif(nB,A,B)
       if(Model== "AlmostFlat") 
            rB <- function() ralmostflat(nB,par[3])
       if(Model== "Beta") 
            rB <- function() rbeta(nB,par[3],par[4])
       x <- numeric()
       repeat {
            x <- c(x,rB())        
            x <- x[x>A]
            x <- x[x<B]
            if(length(x)>=nB) {
                 x <- x[1:nB]
                 break
            }    
       }
       if(nS==0) return(sort(x)) 
       y <- numeric()
       repeat {
            y <- c(y,rnorm(nS,par[1],par[2]))        
            y <- y[y>A]
            y <- y[y<B]
            if(length(y)>=nS) {
                 y <- y[1:nS]
                 break
            }    
       }       
       sort(c(x,y))
}
par.fit <-
function (x,p,fitWhat="All",Model,dBackModel,A=0,B=1,Show=FALSE) 
{
       dBack <- function(x,par) {
             if(Model=="Other") {
                  f <- function(x) eval(parse(text=dBackModel))
                  I <- integrate(f,A,B)$value
                  y <- f(x)/I
             } 
             if(Model=="Exponential") 
                  y <- dexp(x,par[1])/diff(pexp(c(A,B),par[1]))
             if(Model=="Flat") 
                  y <- rep(1/(B-A),length(x))
             if(Model=="Beta") 
                  y <- dbeta(x,par[1],par[2])     
             y
      }            
      dSig <- function(x,p) dnorm(x,p[1],p[2])/diff(pnorm(c(A,B),p[1],p[2]))
      if(fitWhat=="All")
              log.ll <- function(p,x,pfix) -sum(log((1-p[1])*dBack(x,p[-c(1:3)])+p[1]*dSig(x,p[2:3])))
      if(fitWhat=="Mix")        
              log.ll <- function(p,x,pfix) -sum(log((1-p[1])*dBack(x,p[-1])+p[1]*dSig(x,pfix[2:3])))
      if(fitWhat=="MixLoc")        
              log.ll <- function(p,x,pfix) -sum(log((1-p[1])*dBack(x,p[-c(1:2)])+p[1]*dSig(x,c(p[2],pfix[3]))))                 
      if(Show) {
          par(mfrow=c(2,2))
          a<-seq(0,0.2,length=100)
          y<-0*a
          for(i in 1:100) y[i]<-log.ll(c(a[i],p[2]),x,p)
          plot(a,y,type="l")
          a<-seq(0.3,0.7,length=100)
          y<-0*a
          for(i in 1:100) y[i]<-log.ll(c(p[1],a[i]),x,p)
          plot(a,y,type="l")
          
      }
      if(fitWhat=="All" & Model=="Other") {L=c(0,A,0.00001);U=c(0.999,B,100)}            
      if(fitWhat=="All" & Model=="Flat") {L=c(0,A,0.00001);U=c(0.999,B,100)}      
      if(fitWhat=="All" & Model=="Beta") {L=c(0,0,0.00001,0.001,0.0001);U=c(0.999,1,100,100,100)}
      if(fitWhat=="All" & Model=="Exponential") {L=c(0,A,0,0.001);U=c(0.999,B,100,100)}
      if(fitWhat=="MixLoc" & Model=="Other") {L=c(0,A);U=c(0.999,B)}      
      if(fitWhat=="MixLoc" & Model=="Flat") {L=c(0,A);U=c(0.999,B)}
      if(fitWhat=="MixLoc" & Model=="Beta") {L=c(0,0,0.001,0.0001);U=c(0.999,1,100,100)}      
      if(fitWhat=="MixLoc" & Model=="Exponential") {L=c(0,A,0.001);U=c(0.999,B,100)}
      if(fitWhat=="Mix" & Model=="Other") {L=0;U=0.999}
      if(fitWhat=="Mix" & Model=="Flat") {L=0;U=0.999}
      if(fitWhat=="Mix" & Model=="Beta") {L=c(0,0.001,0.0001);U=c(0.999,100,100)}            
      if(fitWhat=="Mix" & Model=="Exponential") {L=c(0,0.0001);U=c(0.999,100)}

      if(fitWhat=="All")      
          out <- optim(p,log.ll,x=x,pfix=p,lower=L,upper=U,hessian=TRUE,method = "L-BFGS-B")    
      if(fitWhat=="Mix") 
          out <- optim(p[-c(2:3)],log.ll,x=x,pfix=p,lower=L,upper=U,hessian=TRUE,method = "L-BFGS-B")    
      if(fitWhat=="MixLoc")      
          out <- optim(p[-3],log.ll,x=x,pfix=p,lower=L,upper=U,hessian=TRUE,method = "L-BFGS-B")    
      par <- out$par    
      hess <- out$hessian
      if(length(p)==3 & fitWhat=="Mix") err<-1/sqrt(hess)
      else err <- sqrt(diag(solve(hess))) 
      alpha <- length(x)*c(par[1],err[1])
      loc <- c(p[2],0)
      backPar <- cbind(par[-1],err[-1])  
      if(fitWhat=="MixLoc" | fitWhat=="All") {
          loc <-  c(par[2],err[2]) 
          if(length(p)>3) backPar <- cbind(par[-c(1:2)],err[-c(1:2)])  
      }    
      width <- c(p[3],0)   
      if(fitWhat=="All") {
          width <-  c(par[3],err[3])   
          if(length(p)>3) backPar <- cbind(par[-c(1:3)],err[-c(1:3)])  
      }
      if(length(p)==3) par <- rbind(alpha,loc,width)    
      else {
          par <- rbind(alpha,loc,width,backPar)    
          rownames(par)[4:length(p)] <- paste("backPar",1:(length(p)-3),sep="")
      }    
      y <- (1-par[1,1]/length(x))*dBack(x,par[-c(1:3),1])+par[1,1]/length(x)*dSig(x,par[2:3,1])
      z <- trw(x,A=A,B=B)
      list(x=x,xf=y,Tx=z$Tx,Txf=y*z$dxdy,par=par)
}
trw <-
function(x,A=-Inf,B=Inf,which=2) {
  if(is.finite(A) && is.infinite(B)) {
        Tx=log(x-A)
        dxdy=exp(Tx)
  }
  if(is.infinite(A) && is.finite(B)) {
        Tx=log(B-x)
        dxdy=-exp(Tx)
  }
  if(is.finite(A) && is.finite(B)) {
    if(which==1) {
      Tx=qnorm((x-A)/(B-A))
      dxdy=dnorm(tx)
    }
    if(which==2) {
      Tx=log((x-A)/(B-x))
      dxdy=(B-A)*exp(Tx)/(exp(Tx)+1)^2
    }
  }  
  if(is.infinite(A) && is.infinite(B)) {
        Tx=x
        dxdy=1    
  }     
  list(Tx=Tx,dxdy=dxdy)
}
bw.nrd <-
function (x) 
{
    if (length(x) < 2L) 
        stop("need at least 2 data points")
    r <- quantile(x, c(0.25, 0.75))
    h <- (r[2L] - r[1L])/1.34
    1.06 * min(sqrt(var(x)), h) * length(x)^(-1/5)
}
sp.fit <-
function (x,p,fitWhat="All",A=0,B=1,Show=FALSE,log.ll.out=FALSE) 
{ 
      z <- trw(x, A, B, 2)
      Tx <- z$Tx
      dxdy <- z$dxdy
      bw <- bw.nrd(Tx)
      dSig <- function(x,p) dnorm(x,p[1],p[2])/diff(pnorm(c(A,B),p[1],p[2])) * dxdy      
      
      if(fitWhat=="All") 
          log.ll <- function(p,x,Tx,pfix) {
              yS <- dSig(x,p[2:3])
              yB <- dnpw(Tx,bw,p[1],yS) 
              I <- c(1:length(x))[yB+yS==0]
              if(length(I)>0) yB[I] <- min(yB[yB>0])
              -sum(log((1-p[1])*yB+p[1]*yS))
          }    
      if(fitWhat=="Mix") {
          yS <- dSig(x,pfix[2:3])
          log.ll <- function(p,x,Tx,pfix) {     
              yB <- dnpw(Tx,bw,p[1],yS) 
              I <- c(1:length(x))[yB+yS==0]
              if(length(I)>0) yB[I] <- min(yB[yB>0])              
              -sum(log((1-p[1])*yB+p[1]*yS))
          }           
      }    
      if(fitWhat=="MixLoc") 
          log.ll <- function(p,x,Tx,pfix) {
              yS <- dSig(x,c(p[2],pfix[3]))
              yB <- dnpw(Tx,bw,p[1],yS) 
              I <- c(1:length(x))[yB+yS==0]
              if(length(I)>0) yB[I] <- min(yB[yB>0])              
              -sum(log((1-p[1])*yB+p[1]*yS))
          }   
      if(log.ll.out) return(log.ll(p,x,Tx,p))    
      if(Show) {
          par(mfrow=c(2,2))
          a <- seq(0.05,0.2,length=30)
          y <- a
          for(i in 1:30) y[i]<-sp.fit(x,c(a[i],p[2:3]),"All",A,B,F,T)
          plot(a,y,type="l")
          a <- seq(p[2]-3*p[3],p[2]+3*p[3],length=30)
          y <- a
          for(i in 1:30) y[i]<-sp.fit(x,c(p[1],a[i],p[3]),"All",A,B,F,T)
          plot(a,y,type="l")      
          a <- seq(0.001,3*p[3],length=30)
          y <- a
          for(i in 1:30) y[i]<-sp.fit(x,c(p[1:2],a[i]),"All",A,B,F,T)
          plot(a,y,type="l")      
          
      }     
      LeftLoc <- max(A+abs(0.01*A),p[2]-3*p[3]) 
      RightLoc <- min(B-abs(0.01*B),p[2]+3*p[3])
      LeftWidth <- max(0.0001,p[3]/3) 
      RightWidth <- 3*p[3]      
      if(fitWhat=="All") {L=c(0.0001,LeftLoc,LeftWidth);U=c(0.999,RightLoc,RightWidth)}      
      if(fitWhat=="MixLoc") {L=c(0.0001,LeftLoc);U=c(0.999,RightLoc)}
      if(fitWhat=="Mix") {L=0.0001;U=0.999}

      if(fitWhat=="All")      
          out <- optim(p,log.ll,x=x,Tx=Tx,pfix=p,lower=L,upper=U,hessian=TRUE,method = "L-BFGS-B")    
      if(fitWhat=="Mix")       
          out <- optim(p[1],log.ll,x=x,Tx=Tx,pfix=p,lower=L,upper=U,hessian=TRUE,method = "L-BFGS-B")    
      if(fitWhat=="MixLoc")      
          out <- optim(p[-3],log.ll,x=x,Tx=Tx,pfix=p,lower=L,upper=U,hessian=TRUE,method = "L-BFGS-B")    
      par <- out$par    
      hess <- out$hessian
      if(fitWhat=="Mix") err<-1/sqrt(hess)
      else err <- sqrt(diag(solve(hess)))
      alpha <- length(x)*c(par[1],err[1])
      loc <- c(p[2],0)
      if(fitWhat=="MixLoc" | fitWhat=="All") 
          loc <-  c(par[2],err[2])
      width <- c(p[3],0)   
      if(fitWhat=="All") 
          width <-  c(par[3],err[3])   
      par <- rbind(alpha,loc,width)  
      yS <- dSig(x,par[2:3,1])
      yB <- dnpw(Tx,bw,p[1],yS) 
      y <- (1-alpha[1]/length(x))*yB+alpha[1]/length(x)*yS  
      list(x=x,xf=y*(B-A)/(x-A)/(B-x),Tx=Tx,Txf=y,par=par)
}
dnpw <-
function (x,bw,alpha,dsig,niter=10) 
{
    n=length(x)
    f=0*x
    if(missing(bw)) bw=bw.nrd(x) 
    w=1-dsig/max(dsig) 
    for(j in 1:niter) {
        g <- density(x,bw=bw,weights=w/sum(w),n=1024)
        f <- approx(g$x,g$y,x)$y
        f=f/sum( (f[-1]+f[-n])/2*(x[-1]-x[-n]) )
        w=(1-alpha)*f/((1-alpha)*f+alpha*dsig)
     }
     f    
}
