require(Rcpp)
shinyUI(fluidPage(
  titlePanel("Semiparametric Fitting Method"),

  sidebarLayout(
      sidebarPanel(
           uiOutput("checkGraph"), 
           actionButton("button",HTML("<font color=\"red\">Go!<font color=\"black\">")),
           selectInput("whichData", "Data?", 
                  choices = c("Included Background Models","Upload your own","Enter your own background function")
           ),
           conditionalPanel( condition = "input.whichData == 'Enter your own background function'",
                  textInput("backfun","Write your background function:",value=" ")
           ),
           conditionalPanel( condition = "input.whichData != 'Upload your own'",
                  conditionalPanel( condition = "input.whichData == 'Included Background Models'",                     
                      fluidRow(column(6,selectInput("Model","Background Model",choices = c("Exponential","Flat","Beta"))),
                                column(6,conditionalPanel( condition = "input.Model == 'Exponential'",
                                             textInput("lambda",HTML("Rate  &lambda;"),value=1,width="100px")
                                         ),  
                                         conditionalPanel( condition = "input.Model == 'Beta'",
                                             fluidRow(column(6,textInput("pbeta1",HTML("&alpha;"),value=1,width="100px")),
                                                     column(6,textInput("pbeta2",HTML("&beta;"),value=1,width="100px"))
                                             )
                                         )
                                )
                      )
                 )
           ),                               
           HTML("<h5><b>Info for Gaussian Signal</b></h5>"),           
           fluidRow(column(5,textInput("mu","Mean",value=0.5)),    
                   column(5,offset=1,textInput("sig","Std Deviation",value=0.025))
           ),  
           conditionalPanel( condition = "input.whichData != 'Upload your own'",
                fluidRow(column(4,textInput("nB","Number of Background Events",value=1000)),     
                    column(4,textInput("nS","Number of Signal Events",value=100)),
                    column(4,textInput("nBy","Pure Background Sample ",value=1000))     
                )
           ),
           conditionalPanel( condition = "input.whichData == 'Upload your own'", 
                textInput("nS1","Starting guess for number of Signal Events",value=100)
           ),    
           conditionalPanel( condition = "input.whichData != 'Upload your own'",
                  HTML("<h5><b>Range of Data</h5>"),          
                  fluidRow(column(5,textInput("A","Left Endpoint",value=0)),
                          column(5,textInput("B","Right Endpoint",value=1))
                  )
           ),
           conditionalPanel( condition = "input.mytabs == 'Bandwidth'",
                  HTML("<h5><b>Choose Range of Bandwidths</h5>"),          
                  fluidRow(column(5,textInput("bw1","Smallest",value=0)),
                          column(5,textInput("bw2","Largest",value=0))
                  )
           ),                               
           conditionalPanel( condition = "input.whichData == 'Upload your own'",
                  fileInput('file1', 'Choose file with signal to upload',
                      accept = c('text/csv','text/comma-separated-values','text/tab-separated-values',
                      'text/plain','.csv','.tsv')
                  ),
                  fileInput('file2', 'Choose background only file to upload',
                          accept = c('text/csv','text/comma-separated-values','text/tab-separated-values',
                          'text/plain','.csv','.tsv')
                  )
           ),    
           selectInput("fitWhat","Fit for ...",
                choices=c("Number of Signal events only",
                          "Number of Signal events and Signal Location",
                          "All three")
           ),
           selectInput("doGof","calculate goodness-of-fit measures",
                choices=c("No","Yes")
           ),               
           conditionalPanel( condition = "input.doGof == 'Yes'",                         
                selectInput("whichGof","GOF for:",
                    choices=c("Semiparametric only","All methods")
                )
            ),                           
            conditionalPanel( condition = "input.mytabs == 'Bootstrap'",
                    selectInput("whichBoot","Bootstrap for:",
                          choices=c("Semiparametric only","All methods")
                    ),
                    numericInput("nBoot","Number of Bootstrap runs",value=500)
            ), 
            conditionalPanel( condition = "input.mytabs == 'Coverage'",
                    selectInput("whichSim","Simulation for:",
                          choices=c("Semiparametric only","All methods")
                    ),
                    numericInput("nSim","Number of MC runs",value=500)
           ),  
           uiOutput("appinfo"),width=4
        ),
  mainPanel(
      tabsetPanel(
          tabPanel("Data",uiOutput("introtext"),
                         uiOutput("text1"),
                        plotOutput("plot1"),
                        uiOutput("text2")),
          tabPanel("Bootstrap",uiOutput("boottext"),
                            plotOutput("bootplot")),
          tabPanel("Coverage",uiOutput("simtext"),
                            plotOutput("simplot"),
                            uiOutput("simtext1")),
          tabPanel("Bandwidth",uiOutput("bandtext"),
                            plotOutput("bandplot"),
                            uiOutput("bandtext1")),
          id="mytabs")
    )
  )    
))

